//============================================================================
//  aPCI-8806 Device Driver I/F Control
//  apci8806ctl.c
//  Author  Shoichiro Ito(ADTEK SYSTEM SCIENCE Co.,Ltd.)
//  Date    2001/03/30
//  Rev.    1.0.0
//============================================================================

#include <stdio.h>	// used printf,perror...
#include <fcntl.h>	// used open
#include <unistd.h>	// used close,usleep
#include <sys/ioctl.h>	// used ioctl
#include <sys/poll.h>	// used poll
#include <string.h>	// used strerror
#include <errno.h>	// used errno

#include "apci8806ctl.h"


/*----------------- Define Device Control ----------------------------------*/

#define APCI8806_FNC_EXEC	1		// ¹
#define APCI8806_FNC_NOTEXEC	0		// ¹Բǽ

typedef struct _apci8806_ctl
{
	int	  fd;
	void*	  arg;
	int	  bexec;			// ǽ¹
} apci8806_ctl, *papci8806_ctl;

static apci8806_ctl ctl[APCI8806_MAX_DEVICES] =
{
	{ 0 }, { 0 }, { 0 }, { 0 }, { 0 }, { 0 }, { 0 }, { 0 },
	{ 0 }, { 0 }, { 0 }, { 0 }, { 0 }, { 0 }, { 0 }, { 0 }
};


/*----------------- Define Static Functions --------------------------------*/

static int dev_open(int log_slot);
static int is_log_slot(int log_slot);
static int is_pass(int log_slot);


/*==========================================================================*/
/*  apci8806_create - ǥХץ					    */
/*  Input : log_slot - åȻ(APCI8806_SLOT_AUTO ξ硢	    */
/*					̤ѥǥХ)		    */
/*  Output: return - 0 ̤ 顼				    */
/*		     0 ʾ å				    */
/*==========================================================================*/
int apci8806_create(int log_slot)
{
	int fd = 0, i;

	// åȻ
	if (log_slot != APCI8806_SLOT_AUTO)
	{
		// åȤ
		if (!(log_slot >= 0 && log_slot < APCI8806_MAX_DEVICES))
			return APCI8806_EINVAL;

		// ǥХץ
		fd = dev_open(log_slot);
		if (fd < 0)
		{
			PDEBUG("Error: open, %s\n", strerror(errno));

			if (errno == EBUSY)
				return APCI8806_EBUSY;
			return APCI8806_ENODEV;
		}

		ctl[log_slot].fd    = fd;
		ctl[log_slot].bexec = APCI8806_FNC_NOTEXEC;

		return log_slot;
	}

	// ̤ѥǥХץ
	for (i = 0; i < APCI8806_MAX_DEVICES; i++)
	{
		// ǥХϥץ
		if (ctl[i].fd != 0)
			continue;

		// ǥХץ
		fd = dev_open(i);
		if (fd < 0)
			continue;

		break;
	}

	// ץǤǥХ̵
	if (i >= APCI8806_MAX_DEVICES)
		return APCI8806_ENODEV;

	ctl[i].fd    = fd;
	ctl[i].bexec = APCI8806_FNC_NOTEXEC;

	return i;		// return log_slot
}

/*==========================================================================*/
/*  apci8806_close - ǥХ					    */
/*  Input : log_slot - åȻ					    */
/*  Output: return - 0 ̤ 顼				    */
/*		     0 						    */
/*==========================================================================*/
int apci8806_close(int log_slot)
{
	// ѥ᡼
	// åȤ
	if (!is_log_slot(log_slot))
		return APCI8806_EINVAL;

	// ǽμ¹Բ
	if (!is_pass(log_slot))
		return APCI8806_EBUSY;

	ctl[log_slot].bexec = APCI8806_FNC_EXEC;		// ¹

	// ǥХΥ
	close(ctl[log_slot].fd);

	ctl[log_slot].fd    = 0;
	ctl[log_slot].bexec = APCI8806_FNC_NOTEXEC;		// ¹Բǽ

	return APCI8806_OK;
}

/*==========================================================================*/
/*  apci8806_set_da_data - D/A ǡ				    */
/*  Input : log_slot - åȻ					    */
/*	    ch	     - D/A ͥ					    */
/*	    data     - D/A ǡ					    */
/*  Output: return - 0 ̤ 顼				    */
/*		     0 						    */
/*==========================================================================*/
int apci8806_set_da_data(int log_slot, unsigned short ch, unsigned short data)
{
	apci8806_da da;

	// ѥ᡼
	// åȤ
	if (!is_log_slot(log_slot))
		return APCI8806_EINVAL;

	// ǽμ¹Բ
	if (!is_pass(log_slot))
		return APCI8806_EBUSY;

	ctl[log_slot].bexec = APCI8806_FNC_EXEC;		// ¹

	da.value = data;
	da.ch    = ch;

	// ioctl ¹
	if (ioctl(ctl[log_slot].fd, APCI8806_SET_DA_DATA, &da) < 0)
	{
		PDEBUG("Error: ioctl, %s\n", strerror(errno));

		ctl[log_slot].bexec = APCI8806_FNC_NOTEXEC;	// ¹Բǽ
		return APCI8806_ESYSCALL;
	}

	ctl[log_slot].bexec = APCI8806_FNC_NOTEXEC;		// ¹Բǽ

	return APCI8806_OK;
}

/*==========================================================================*/
/*  apci8806_get_exin - ϥݡȤǡ			    */
/*  Input : log_slot - åȻ					    */
/*	    pexin    - ϥݡȥǡǼΰ			    */
/*  Output: return - 0 ̤ 顼				    */
/*		     0 						    */
/*==========================================================================*/
int apci8806_get_exin(int log_slot, unsigned char* pexin)
{
	// ѥ᡼
	// åȤ
	if (!is_log_slot(log_slot))
		return APCI8806_EINVAL;

	if (pexin == NULL)
		return APCI8806_EINVAL;

	// ǽμ¹Բ
	if (!is_pass(log_slot))
		return APCI8806_EBUSY;

	ctl[log_slot].bexec = APCI8806_FNC_EXEC;		// ¹

	// ioctl ¹
	if (ioctl(ctl[log_slot].fd, APCI8806_GET_EXIN, pexin) < 0)
	{
		PDEBUG("Error: ioctl, %s\n", strerror(errno));

		ctl[log_slot].bexec = APCI8806_FNC_NOTEXEC;	// ¹Բǽ
		return APCI8806_ESYSCALL;
	}

	ctl[log_slot].bexec = APCI8806_FNC_NOTEXEC;		// ¹Բǽ

	return APCI8806_OK;
}

/*--------------------------------------------------------------------------*/
/*  dev_open - ǥХΥץ					    */
/*  Input : log_slot - åȻ					    */
/*  Output: return - open 					    */
/*--------------------------------------------------------------------------*/
static int dev_open(int log_slot)
{
	char szbuf[30];

	sprintf(szbuf, "/dev/apci8806%d", log_slot);

	return open(szbuf, O_RDWR);
}

/*--------------------------------------------------------------------------*/
/*  is_log_slot - åȥå					    */
/*  Input : log_slot - åȻ					    */
/*  Output: return - 0 ̵						    */
/*		     1 						    */
/*--------------------------------------------------------------------------*/
static int is_log_slot(int log_slot)
{
	// ϰϤ
	if (!(log_slot >= 0 && log_slot < APCI8806_MAX_DEVICES))
		return 0;

	// ǥХξ
	if (ctl[log_slot].fd == 0)
		return 0;

	// Ѳǽ
	return 1;
}

/*--------------------------------------------------------------------------*/
/*  is_pass - ¹Բǽå						    */
/*  Input : log_slot - åȻ					    */
/*  Output: return - 0 ̵						    */
/*		     1 						    */
/*--------------------------------------------------------------------------*/
static int is_pass(int log_slot)
{
	// ǽμ¹Ծ
	if (ctl[log_slot].bexec != APCI8806_FNC_NOTEXEC)
		return 0;	// ¹Բ

	// ¹Բǽ
	return 1;
}
